package com.ruoyi.system.utils.jsch;

import com.jcraft.jsch.JSchException;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import com.jcraft.jsch.ChannelExec;
import com.ruoyi.common.core.utils.StringUtils;
import java.nio.charset.StandardCharsets;
import java.io.IOException;
import java.util.Objects;
import com.jcraft.jsch.UserInfo;
import com.jcraft.jsch.JSch;
import com.ruoyi.system.kernel.ServerLogKernel;
import java.io.OutputStream;
import java.io.InputStream;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.Session;

public class ServiceShell
{
    private Session session;
    private Channel channel;
    private InputStream in;
    private OutputStream out;
    private static final int IDLE_TIMEOUT_MINUTES = 600;
    private static String COMMAND_PROMPT;
    private Long serverId;
    private ServerLogKernel serverLogKernel;
    
    public ServiceShell() {
        this.session = null;
        this.channel = null;
        this.in = null;
        this.out = null;
        this.serverId = null;
        this.serverLogKernel = null;
    }
    
    public ServiceShell(final Long serverId, final ServerLogKernel serverLogKernel) {
        this.session = null;
        this.channel = null;
        this.in = null;
        this.out = null;
        this.serverId = null;
        this.serverLogKernel = null;
        this.serverId = serverId;
        this.serverLogKernel = serverLogKernel;
    }
    
    public void connect(final String host, final String username, final String password, final int port) {
        try {
            final JSch jSch = new JSch();
            (this.session = jSch.getSession(username, host, port)).setPassword(password);
            this.session.setUserInfo((UserInfo)new MyUserInfo());
            this.session.connect(30000);
            this.channel = this.session.openChannel("shell");
            this.in = this.channel.getInputStream();
            this.out = this.channel.getOutputStream();
            this.channel.connect(3000);
            if (!Objects.equals(username, "root")) {
                this.sendCommand("sudo su -");
            }
            else {
                this.readCommandOutput("", null);
            }
        }
        catch (final Exception e) {
            throw new RuntimeException("连接服务器失败", e);
        }
    }
    
    public String sendCommand(final String command) {
        try {
            this.send(command + "\n");
            return this.readCommandOutput(command, null);
        }
        catch (final IOException e) {
            throw new RuntimeException("Failed to send command", e);
        }
    }
    
    public String sendCommand(final String command, final Integer timeoutMinute) {
        try {
            this.send(command + "\n");
            return this.readCommandOutput(command, timeoutMinute);
        }
        catch (final IOException e) {
            throw new RuntimeException("Failed to send command", e);
        }
    }
    
    private String readCommandOutput(final String command, Integer timeoutMinute) throws IOException {
        if (timeoutMinute == null || timeoutMinute == 0) {
            timeoutMinute = 2;
        }
        final int timeout = timeoutMinute * 600;
        final String outStr = "空闲弹出，" + timeoutMinute + "分钟";
        final StringBuilder output = new StringBuilder();
        final byte[] buffer = new byte[1024];
        int num = 0;
        boolean endOfCommand = false;
        while (!endOfCommand) {
            if (this.in.available() <= 0) {
                ++num;
            }
            else {
                num = 0;
            }
            if (num > timeout) {
                System.out.println(outStr);
                output.append(outStr);
                break;
            }
            while (this.in.available() > 0) {
                final int bytesRead = this.in.read(buffer, 0, 1024);
                if (bytesRead < 0) {
                    endOfCommand = true;
                    break;
                }
                final String part = new String(buffer, 0, bytesRead, StandardCharsets.UTF_8);
                if (StringUtils.isEmpty(part)) {
                    ++num;
                }
                else {
                    num = 0;
                }
                if (num > timeout) {
                    System.out.println(outStr);
                    output.append(outStr);
                    endOfCommand = true;
                    break;
                }
                if (StringUtils.isEmpty(part)) {
                    continue;
                }
                output.append(part);
                if (this.isEndOfCommand(part)) {
                    endOfCommand = true;
                    break;
                }
                if (!command.startsWith("sudo certbot certonly")) {
                    continue;
                }
                this.handleCertbotPrompts(part);
            }
            if (endOfCommand) {
                break;
            }
            try {
                Thread.sleep(100L);
                continue;
            }
            catch (final InterruptedException e) {
                Thread.currentThread().interrupt();
                throw new RuntimeException("Thread interrupted", e);
            }
            break;
        }
        return output.toString();
    }
    
    private boolean isEndOfCommand(final String part) {
        return part.contains("root@") && (part.contains(":~$") || part.contains(":~#") || part.contains("~]#") || part.contains(ServiceShell.COMMAND_PROMPT));
    }
    
    private void handleCertbotPrompts(final String part) throws IOException {
        if (part.contains("Select the appropriate number [1-2]")) {
            this.send("2\n");
        }
        if (part.contains("(Y)es/(N)o")) {
            this.send("N\n");
        }
    }
    
    private void send(final String command) throws IOException {
        if (this.out != null) {
            this.out.write(command.getBytes());
            this.out.flush();
        }
    }
    
    public String executeCommand(final String command) {
        final StringBuilder output = new StringBuilder();
        Channel channel = null;
        try {
            channel = this.session.openChannel("exec");
            ((ChannelExec)channel).setCommand(command);
            channel.setInputStream((InputStream)null);
            channel.connect();
            try (final InputStream in = channel.getInputStream();
                 final BufferedReader reader = new BufferedReader(new InputStreamReader(in))) {
                String line;
                while ((line = reader.readLine()) != null) {
                    output.append(line).append("\n");
                }
            }
        }
        catch (final IOException | JSchException e) {
            return "";
        }
        finally {
            if (channel != null && channel.isConnected()) {
                channel.disconnect();
            }
        }
        return output.toString().trim();
    }
    
    public void disconnect() {
        if (this.channel != null) {
            this.channel.disconnect();
        }
        if (this.session != null) {
            this.session.disconnect();
        }
    }
    
    public static void main(final String[] args) {
        final ServiceShell shell = new ServiceShell();
        try {
            shell.connect("43.153.3.62", "ubuntu", "931yELfCHEwT", 22);
            final String logs = shell.executeCommand("sudo docker logs --tail 100 sync-mysql");
            System.out.println(logs);
            shell.disconnect();
        }
        catch (final Exception e) {
            System.out.println(e.getMessage());
            shell.disconnect();
        }
    }
    
    static {
        ServiceShell.COMMAND_PROMPT = "command_send_success";
    }
}
